@@ -8,8 +8,7 @@
  * Copyright(c) 2012 - 2014 Intel Corporation. All rights reserved.
  * Copyright(c) 2013 - 2015 Intel Mobile Communications GmbH
  * Copyright(c) 2016 - 2017 Intel Deutschland GmbH
- * Copyright(c) 2018        Intel Corporation
- * Copyright (C) 2018 Intel Corporation
+ * Copyright (C) 2018 - 2019 Intel Corporation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -32,8 +31,7 @@
  * Copyright(c) 2012 - 2014 Intel Corporation. All rights reserved.
  * Copyright(c) 2013 - 2015 Intel Mobile Communications GmbH
  * Copyright(c) 2016 - 2017 Intel Deutschland GmbH
- * Copyright(c) 2018        Intel Corporation
- * Copyright (C) 2018 Intel Corporation
+ * Copyright (C) 2018 - 2019 Intel Corporation
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -69,13 +67,47 @@
 #include "mvm.h"
 #include "iwl-vendor-cmd.h"
 
-#ifdef CPTCFG_IWLWIFI_LTE_COEX
-#include "lte-coex.h"
-#endif
-
 #include "iwl-io.h"
 #include "iwl-prph.h"
 
+static LIST_HEAD(device_list);
+static DEFINE_SPINLOCK(device_list_lock);
+
+static int iwl_mvm_netlink_notifier(struct notifier_block *nb,
+				    unsigned long state,
+				    void *_notify)
+{
+	struct netlink_notify *notify = _notify;
+	struct iwl_mvm *mvm;
+
+	if (state != NETLINK_URELEASE || notify->protocol != NETLINK_GENERIC)
+		return NOTIFY_DONE;
+
+	spin_lock_bh(&device_list_lock);
+	list_for_each_entry(mvm, &device_list, list) {
+		if (mvm->csi_portid == netlink_notify_portid(notify))
+			mvm->csi_portid = 0;
+	}
+	spin_unlock_bh(&device_list_lock);
+
+	return NOTIFY_OK;
+}
+
+static struct notifier_block iwl_mvm_netlink_notifier_block = {
+	.notifier_call = iwl_mvm_netlink_notifier,
+};
+
+void iwl_mvm_vendor_cmd_init(void)
+{
+	WARN_ON(netlink_register_notifier(&iwl_mvm_netlink_notifier_block));
+	spin_lock_init(&device_list_lock);
+}
+
+void iwl_mvm_vendor_cmd_exit(void)
+{
+	netlink_unregister_notifier(&iwl_mvm_netlink_notifier_block);
+}
+
 static const struct nla_policy
 iwl_mvm_vendor_attr_policy[NUM_IWL_MVM_VENDOR_ATTR] = {
 	[IWL_MVM_VENDOR_ATTR_LOW_LATENCY] = { .type = NLA_FLAG },
@@ -109,14 +141,26 @@ iwl_mvm_vendor_attr_policy[NUM_IWL_MVM_VENDOR_ATTR] = {
 	[IWL_MVM_VENDOR_ATTR_FIPS_TEST_VECTOR_HW_AES] = { .type = NLA_NESTED },
 };
 
-static int iwl_mvm_parse_vendor_data(struct nlattr **tb,
-				     const void *data, int data_len)
+static struct nlattr **iwl_mvm_parse_vendor_data(const void *data, int data_len)
 {
-	if (!data)
-		return -EINVAL;
+	struct nlattr **tb;
+	int err;
 
-	return nla_parse(tb, MAX_IWL_MVM_VENDOR_ATTR, data, data_len,
-			 iwl_mvm_vendor_attr_policy, NULL);
+	if (!data)
+		return ERR_PTR(-EINVAL);
+
+	tb = kcalloc(MAX_IWL_MVM_VENDOR_ATTR + 1, sizeof(*tb), GFP_KERNEL);
+	if (!tb)
+		return ERR_PTR(-ENOMEM);
+
+	err = nla_parse(tb, MAX_IWL_MVM_VENDOR_ATTR, data, data_len,
+			iwl_mvm_vendor_attr_policy, NULL);
+	if (err) {
+		kfree(tb);
+		return ERR_PTR(err);
+	}
+
+	return tb;
 }
 
 static int iwl_mvm_set_low_latency(struct wiphy *wiphy,
@@ -125,7 +169,7 @@ static int iwl_mvm_set_low_latency(struct wiphy *wiphy,
 {
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	int err;
 	struct ieee80211_vif *vif = wdev_to_ieee80211_vif(wdev);
 	bool low_latency;
@@ -134,10 +178,11 @@ static int iwl_mvm_set_low_latency(struct wiphy *wiphy,
 		return -ENODEV;
 
 	if (data) {
-		err = iwl_mvm_parse_vendor_data(tb, data, data_len);
-		if (err)
-			return err;
+		tb = iwl_mvm_parse_vendor_data(data, data_len);
+		if (IS_ERR(tb))
+			return PTR_ERR(tb);
 		low_latency = tb[IWL_MVM_VENDOR_ATTR_LOW_LATENCY];
+		kfree(tb);
 	} else {
 		low_latency = false;
 	}
@@ -179,7 +224,7 @@ static int iwl_mvm_set_country(struct wiphy *wiphy,
 			       const void *data, int data_len)
 {
 	struct ieee80211_regdomain *regd;
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
 	int retval;
@@ -187,12 +232,14 @@ static int iwl_mvm_set_country(struct wiphy *wiphy,
 	if (!iwl_mvm_is_lar_supported(mvm))
 		return -EOPNOTSUPP;
 
-	retval = iwl_mvm_parse_vendor_data(tb, data, data_len);
-	if (retval)
-		return retval;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
-	if (!tb[IWL_MVM_VENDOR_ATTR_COUNTRY])
-		return -EINVAL;
+	if (!tb[IWL_MVM_VENDOR_ATTR_COUNTRY]) {
+		retval = -EINVAL;
+		goto free;
+	}
 
 	mutex_lock(&mvm->mutex);
 
@@ -211,379 +258,31 @@ static int iwl_mvm_set_country(struct wiphy *wiphy,
 	kfree(regd);
 unlock:
 	mutex_unlock(&mvm->mutex);
+free:
+	kfree(tb);
 	return retval;
 }
 
-#ifdef CPTCFG_IWLWIFI_LTE_COEX
-static int iwl_vendor_lte_coex_state_cmd(struct wiphy *wiphy,
-					 struct wireless_dev *wdev,
-					 const void *data, int data_len)
-{
-	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
-	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	const struct lte_coex_state_cmd *cmd = data;
-	struct sk_buff *skb;
-	int err = LTE_OK;
-
-	skb = cfg80211_vendor_cmd_alloc_reply_skb(wiphy, 100);
-	if (!skb)
-		return -ENOMEM;
-
-	if (data_len != sizeof(*cmd)) {
-		err = LTE_INVALID_DATA;
-		goto out;
-	}
-
-	IWL_DEBUG_COEX(mvm, "LTE-COEX: state cmd:\n\tstate: %d\n",
-		       cmd->lte_state);
-
-	switch (cmd->lte_state) {
-	case LTE_OFF:
-		if (mvm->lte_state.has_config &&
-		    mvm->lte_state.state != LTE_CONNECTED) {
-			err = LTE_STATE_ERR;
-			goto out;
-		}
-		mvm->lte_state.state = LTE_OFF;
-		mvm->lte_state.has_config = 0;
-		mvm->lte_state.has_rprtd_chan = 0;
-		mvm->lte_state.has_sps = 0;
-		mvm->lte_state.has_ft = 0;
-		break;
-	case LTE_IDLE:
-		if (!mvm->lte_state.has_static ||
-		    (mvm->lte_state.has_config &&
-		     mvm->lte_state.state != LTE_CONNECTED)) {
-			err = LTE_STATE_ERR;
-			goto out;
-		}
-		mvm->lte_state.has_config = 0;
-		mvm->lte_state.has_sps = 0;
-		mvm->lte_state.state = LTE_IDLE;
-		break;
-	case LTE_CONNECTED:
-		if (!(mvm->lte_state.has_config)) {
-			err = LTE_STATE_ERR;
-			goto out;
-		}
-		mvm->lte_state.state = LTE_CONNECTED;
-		break;
-	default:
-		err = LTE_ILLEGAL_PARAMS;
-		goto out;
-	}
-
-	mvm->lte_state.config.lte_state = cpu_to_le32(mvm->lte_state.state);
-
-	mutex_lock(&mvm->mutex);
-	if (iwl_mvm_send_lte_coex_config_cmd(mvm))
-		err = LTE_OTHER_ERR;
-	mutex_unlock(&mvm->mutex);
-
-out:
-	if (err)
-		iwl_mvm_reset_lte_state(mvm);
-
-	if (nla_put_u8(skb, NLA_BINARY, err)) {
-		kfree_skb(skb);
-		return -ENOBUFS;
-	}
-
-	return cfg80211_vendor_cmd_reply(skb);
-}
-
-static int iwl_vendor_lte_coex_config_cmd(struct wiphy *wiphy,
-					  struct wireless_dev *wdev,
-					  const void *data, int data_len)
-{
-	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
-	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	const struct lte_coex_config_info_cmd *cmd = data;
-	struct iwl_lte_coex_static_params_cmd *stat = &mvm->lte_state.stat;
-	struct sk_buff *skb;
-	int err = LTE_OK;
-	int i, j;
-
-	skb = cfg80211_vendor_cmd_alloc_reply_skb(wiphy, 100);
-	if (!skb)
-		return -ENOMEM;
-
-	if (data_len != sizeof(*cmd)) {
-		err = LTE_INVALID_DATA;
-		goto out;
-	}
-
-	IWL_DEBUG_COEX(mvm, "LTE-COEX: config cmd:\n");
-
-	/* send static config only once in the FW life */
-	if (mvm->lte_state.has_static)
-		goto out;
-
-	for (i = 0; i < LTE_MWS_CONF_LENGTH; i++) {
-		IWL_DEBUG_COEX(mvm, "\tmws config data[%d]: %d\n", i,
-			       cmd->mws_conf_data[i]);
-		stat->mfu_config[i] = cpu_to_le32(cmd->mws_conf_data[i]);
-	}
-
-	if (cmd->safe_power_table[0] != LTE_SAFE_PT_FIRST ||
-	    cmd->safe_power_table[LTE_SAFE_PT_LENGTH - 1] !=
-							LTE_SAFE_PT_LAST) {
-		err = LTE_ILLEGAL_PARAMS;
-		goto out;
-	}
-
-	/* power table must be ascending ordered */
-	j = LTE_SAFE_PT_FIRST;
-	for (i = 0; i < LTE_SAFE_PT_LENGTH; i++) {
-		IWL_DEBUG_COEX(mvm, "\tsafe power table[%d]: %d\n", i,
-			       cmd->safe_power_table[i]);
-		if (cmd->safe_power_table[i] < j) {
-			err = LTE_ILLEGAL_PARAMS;
-			goto out;
-		}
-		j = cmd->safe_power_table[i];
-		stat->tx_power_in_dbm[i] = cmd->safe_power_table[i];
-	}
-
-	mutex_lock(&mvm->mutex);
-	if (iwl_mvm_send_lte_coex_static_params_cmd(mvm))
-		err = LTE_OTHER_ERR;
-	else
-		mvm->lte_state.has_static = 1;
-	mutex_unlock(&mvm->mutex);
-
-out:
-	if (err)
-		iwl_mvm_reset_lte_state(mvm);
-
-	if (nla_put_u8(skb, NLA_BINARY, err)) {
-		kfree_skb(skb);
-		return -ENOBUFS;
-	}
-
-	return cfg80211_vendor_cmd_reply(skb);
-}
-
-static int in_range(int val, int min, int max)
-{
-	return (val >= min) && (val <= max);
-}
-
-static bool is_valid_lte_range(u16 min, u16 max)
-{
-	return (min == 0 && max == 0) ||
-	       (max >= min &&
-		in_range(min, LTE_FRQ_MIN, LTE_FRQ_MAX) &&
-		in_range(max, LTE_FRQ_MIN, LTE_FRQ_MAX));
-}
-
-static int iwl_vendor_lte_coex_dynamic_info_cmd(struct wiphy *wiphy,
-						struct wireless_dev *wdev,
-						const void *data, int data_len)
-{
-	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
-	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	const struct lte_coex_dynamic_info_cmd *cmd = data;
-	struct iwl_lte_coex_config_cmd *config = &mvm->lte_state.config;
-	struct sk_buff *skb;
-	int err = LTE_OK;
-	int i;
-
-	skb = cfg80211_vendor_cmd_alloc_reply_skb(wiphy, 100);
-	if (!skb)
-		return -ENOMEM;
-
-	if (data_len != sizeof(*cmd)) {
-		err = LTE_INVALID_DATA;
-		goto out;
-	}
-
-	if (!mvm->lte_state.has_static ||
-	    (mvm->lte_state.has_config &&
-	     mvm->lte_state.state != LTE_CONNECTED)) {
-		err = LTE_STATE_ERR;
-		goto out;
-	}
-
-	IWL_DEBUG_COEX(mvm, "LTE-COEX: dynamic cmd:\n"
-		       "\tlte band[0]: %d, chan[0]: %d\n\ttx range: %d - %d\n"
-		       "\trx range: %d - %d\n", cmd->lte_connected_bands[0],
-		       cmd->lte_connected_bands[1], cmd->wifi_tx_safe_freq_min,
-		       cmd->wifi_tx_safe_freq_max, cmd->wifi_rx_safe_freq_min,
-		       cmd->wifi_rx_safe_freq_max);
-
-	/* TODO: validate lte connected bands and channel, and frame struct */
-	config->lte_band = cpu_to_le32(cmd->lte_connected_bands[0]);
-	config->lte_chan = cpu_to_le32(cmd->lte_connected_bands[1]);
-	for (i = 0; i < LTE_FRAME_STRUCT_LENGTH; i++) {
-		IWL_DEBUG_COEX(mvm, "\tframe structure[%d]: %d\n", i,
-			       cmd->lte_frame_structure[i]);
-		config->lte_frame_structure[i] =
-				cpu_to_le32(cmd->lte_frame_structure[i]);
-	}
-	if (!is_valid_lte_range(cmd->wifi_tx_safe_freq_min,
-				cmd->wifi_tx_safe_freq_max) ||
-	    !is_valid_lte_range(cmd->wifi_rx_safe_freq_min,
-				cmd->wifi_rx_safe_freq_max)) {
-		err = LTE_ILLEGAL_PARAMS;
-		goto out;
-	}
-	config->tx_safe_freq_min = cpu_to_le32(cmd->wifi_tx_safe_freq_min);
-	config->tx_safe_freq_max = cpu_to_le32(cmd->wifi_tx_safe_freq_max);
-	config->rx_safe_freq_min = cpu_to_le32(cmd->wifi_rx_safe_freq_min);
-	config->rx_safe_freq_max = cpu_to_le32(cmd->wifi_rx_safe_freq_max);
-	for (i = 0; i < LTE_TX_POWER_LENGTH; i++) {
-		IWL_DEBUG_COEX(mvm, "\twifi max tx power[%d]: %d\n", i,
-			       cmd->wifi_max_tx_power[i]);
-		if (!in_range(cmd->wifi_max_tx_power[i], LTE_MAX_TX_MIN,
-			      LTE_MAX_TX_MAX)) {
-			err = LTE_ILLEGAL_PARAMS;
-			goto out;
-		}
-		config->max_tx_power[i] = cmd->wifi_max_tx_power[i];
-	}
-
-	mvm->lte_state.has_config = 1;
-
-	if (mvm->lte_state.state == LTE_CONNECTED) {
-		mutex_lock(&mvm->mutex);
-		if (iwl_mvm_send_lte_coex_config_cmd(mvm))
-			err = LTE_OTHER_ERR;
-		mutex_unlock(&mvm->mutex);
-	}
-out:
-	if (err)
-		iwl_mvm_reset_lte_state(mvm);
-
-	if (nla_put_u8(skb, NLA_BINARY, err)) {
-		kfree_skb(skb);
-		return -ENOBUFS;
-	}
-
-	return cfg80211_vendor_cmd_reply(skb);
-}
-
-static int iwl_vendor_lte_sps_cmd(struct wiphy *wiphy,
-				  struct wireless_dev *wdev, const void *data,
-				  int data_len)
-{
-	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
-	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	const struct lte_coex_sps_info_cmd *cmd = data;
-	struct iwl_lte_coex_sps_cmd *sps = &mvm->lte_state.sps;
-	struct sk_buff *skb;
-	int err = LTE_OK;
-
-	skb = cfg80211_vendor_cmd_alloc_reply_skb(wiphy, 100);
-	if (!skb)
-		return -ENOMEM;
-
-	if (data_len != sizeof(*cmd)) {
-		err = LTE_INVALID_DATA;
-		goto out;
-	}
-
-	IWL_DEBUG_COEX(mvm, "LTE-COEX: sps cmd:\n\tsps info: %d\n",
-		       cmd->sps_info);
-
-	if (mvm->lte_state.state != LTE_CONNECTED) {
-		err = LTE_STATE_ERR;
-		goto out;
-	}
-
-	/* TODO: validate SPS */
-	sps->lte_semi_persistent_info = cpu_to_le32(cmd->sps_info);
-
-	mutex_lock(&mvm->mutex);
-	if (iwl_mvm_send_lte_sps_cmd(mvm))
-		err = LTE_OTHER_ERR;
-	else
-		mvm->lte_state.has_sps = 1;
-	mutex_unlock(&mvm->mutex);
-
-out:
-	if (err)
-		iwl_mvm_reset_lte_state(mvm);
-
-	if (nla_put_u8(skb, NLA_BINARY, err)) {
-		kfree_skb(skb);
-		return -ENOBUFS;
-	}
-
-	return cfg80211_vendor_cmd_reply(skb);
-}
-
-static int
-iwl_vendor_lte_coex_wifi_reported_channel_cmd(struct wiphy *wiphy,
-					      struct wireless_dev *wdev,
-					      const void *data, int data_len)
-{
-	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
-	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	const struct lte_coex_wifi_reported_chan_cmd *cmd = data;
-	struct iwl_lte_coex_wifi_reported_channel_cmd *rprtd_chan =
-						&mvm->lte_state.rprtd_chan;
-	struct sk_buff *skb;
-	int err = LTE_OK;
-
-	skb = cfg80211_vendor_cmd_alloc_reply_skb(wiphy, 100);
-	if (!skb)
-		return -ENOMEM;
-
-	if (data_len != sizeof(*cmd)) {
-		err = LTE_INVALID_DATA;
-		goto out;
-	}
-
-	IWL_DEBUG_COEX(mvm, "LTE-COEX: wifi reported channel cmd:\n"
-		       "\tchannel: %d, bandwidth: %d\n", cmd->chan,
-		       cmd->bandwidth);
-
-	if (!in_range(cmd->chan, LTE_RC_CHAN_MIN, LTE_RC_CHAN_MAX) ||
-	    !in_range(cmd->bandwidth, LTE_RC_BW_MIN, LTE_RC_BW_MAX)) {
-		err = LTE_ILLEGAL_PARAMS;
-		goto out;
-	}
-
-	rprtd_chan->channel = cpu_to_le32(cmd->chan);
-	rprtd_chan->bandwidth = cpu_to_le32(cmd->bandwidth);
-
-	mutex_lock(&mvm->mutex);
-	if (iwl_mvm_send_lte_coex_wifi_reported_channel_cmd(mvm))
-		err = LTE_OTHER_ERR;
-	else
-		mvm->lte_state.has_rprtd_chan = 1;
-	mutex_unlock(&mvm->mutex);
-
-out:
-	if (err)
-		iwl_mvm_reset_lte_state(mvm);
-
-	if (nla_put_u8(skb, NLA_BINARY, err)) {
-		kfree_skb(skb);
-		return -ENOBUFS;
-	}
-
-	return cfg80211_vendor_cmd_reply(skb);
-}
-#endif /* CPTCFG_IWLWIFI_LTE_COEX */
-
 static int iwl_vendor_frame_filter_cmd(struct wiphy *wiphy,
 				       struct wireless_dev *wdev,
 				       const void *data, int data_len)
 {
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	struct ieee80211_vif *vif = wdev_to_ieee80211_vif(wdev);
-	int err = iwl_mvm_parse_vendor_data(tb, data, data_len);
 
-	if (err)
-		return err;
 	if (!vif)
 		return -EINVAL;
+
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
+
 	vif->filter_grat_arp_unsol_na =
 		tb[IWL_MVM_VENDOR_ATTR_FILTER_ARP_NA];
 	vif->filter_gtk = tb[IWL_MVM_VENDOR_ATTR_FILTER_GTK];
 
+	kfree(tb);
+
 	return 0;
 }
 
@@ -592,23 +291,26 @@ static int iwl_vendor_tdls_peer_cache_add(struct wiphy *wiphy,
 					  struct wireless_dev *wdev,
 					  const void *data, int data_len)
 {
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
 	struct iwl_mvm_tdls_peer_counter *cnt;
 	u8 *addr;
 	struct ieee80211_vif *vif = wdev_to_ieee80211_vif(wdev);
-	int err = iwl_mvm_parse_vendor_data(tb, data, data_len);
-
-	if (err)
-		return err;
+	int err;
 
 	if (!vif)
 		return -ENODEV;
 
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
+
 	if (vif->type != NL80211_IFTYPE_STATION ||
-	    !tb[IWL_MVM_VENDOR_ATTR_ADDR])
-		return -EINVAL;
+	    !tb[IWL_MVM_VENDOR_ATTR_ADDR]) {
+		err = -EINVAL;
+		goto free;
+	}
 
 	mutex_lock(&mvm->mutex);
 	if (mvm->tdls_peer_cache_cnt >= IWL_MVM_TDLS_CNT_MAX_PEERS) {
@@ -640,8 +342,12 @@ static int iwl_vendor_tdls_peer_cache_add(struct wiphy *wiphy,
 	list_add_tail_rcu(&cnt->list, &mvm->tdls_peer_cache_list);
 	mvm->tdls_peer_cache_cnt++;
 
+	err = 0;
+
 out_unlock:
 	mutex_unlock(&mvm->mutex);
+free:
+	kfree(tb);
 	return err;
 }
 
@@ -649,18 +355,21 @@ static int iwl_vendor_tdls_peer_cache_del(struct wiphy *wiphy,
 					  struct wireless_dev *wdev,
 					  const void *data, int data_len)
 {
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
 	struct iwl_mvm_tdls_peer_counter *cnt;
 	u8 *addr;
-	int err = iwl_mvm_parse_vendor_data(tb, data, data_len);
+	int err;
 
-	if (err)
-		return err;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
-	if (!tb[IWL_MVM_VENDOR_ATTR_ADDR])
-		return -EINVAL;
+	if (!tb[IWL_MVM_VENDOR_ATTR_ADDR]) {
+		err = -EINVAL;
+		goto free;
+	}
 
 	addr = nla_data(tb[IWL_MVM_VENDOR_ATTR_ADDR]);
 
@@ -677,10 +386,13 @@ static int iwl_vendor_tdls_peer_cache_del(struct wiphy *wiphy,
 	mvm->tdls_peer_cache_cnt--;
 	list_del_rcu(&cnt->list);
 	kfree_rcu(cnt, rcu_head);
+	err = 0;
 
 out_unlock:
 	rcu_read_unlock();
 	mutex_unlock(&mvm->mutex);
+free:
+	kfree(tb);
 	return err;
 }
 
@@ -688,23 +400,29 @@ static int iwl_vendor_tdls_peer_cache_query(struct wiphy *wiphy,
 					    struct wireless_dev *wdev,
 					    const void *data, int data_len)
 {
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
 	struct iwl_mvm_tdls_peer_counter *cnt;
 	struct sk_buff *skb;
 	u32 rx_bytes, tx_bytes;
 	u8 *addr;
-	int err = iwl_mvm_parse_vendor_data(tb, data, data_len);
+	int err;
 
-	if (err)
-		return err;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
-	if (!tb[IWL_MVM_VENDOR_ATTR_ADDR])
+	if (!tb[IWL_MVM_VENDOR_ATTR_ADDR]) {
+		kfree(tb);
 		return -EINVAL;
+	}
 
 	addr = nla_data(tb[IWL_MVM_VENDOR_ATTR_ADDR]);
 
+	/* we can free the tb, the addr pointer is still valid into the msg */
+	kfree(tb);
+
 	rcu_read_lock();
 	cnt = iwl_mvm_tdls_peer_cache_find(mvm, addr);
 	if (!cnt) {
@@ -718,6 +436,7 @@ static int iwl_vendor_tdls_peer_cache_query(struct wiphy *wiphy,
 		rx_bytes = 0;
 		for (q = 0; q < mvm->trans->num_rx_queues; q++)
 			rx_bytes += cnt->rx[q].bytes;
+		err = 0;
 	}
 	rcu_read_unlock();
 	if (err)
@@ -751,35 +470,41 @@ static int iwl_vendor_set_nic_txpower_limit(struct wiphy *wiphy,
 		.v5.v3.dev_52_low = cpu_to_le16(IWL_DEV_MAX_TX_POWER),
 		.v5.v3.dev_52_high = cpu_to_le16(IWL_DEV_MAX_TX_POWER),
 	};
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	int len = sizeof(cmd);
 	int err;
 
-	err = iwl_mvm_parse_vendor_data(tb, data, data_len);
-	if (err)
-		return err;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
 	if (tb[IWL_MVM_VENDOR_ATTR_TXP_LIMIT_24]) {
 		s32 txp = nla_get_u32(tb[IWL_MVM_VENDOR_ATTR_TXP_LIMIT_24]);
 
-		if (txp < 0 || txp > IWL_DEV_MAX_TX_POWER)
-			return -EINVAL;
+		if (txp < 0 || txp > IWL_DEV_MAX_TX_POWER) {
+			err = -EINVAL;
+			goto free;
+		}
 		cmd.v5.v3.dev_24 = cpu_to_le16(txp);
 	}
 
 	if (tb[IWL_MVM_VENDOR_ATTR_TXP_LIMIT_52L]) {
 		s32 txp = nla_get_u32(tb[IWL_MVM_VENDOR_ATTR_TXP_LIMIT_52L]);
 
-		if (txp < 0 || txp > IWL_DEV_MAX_TX_POWER)
-			return -EINVAL;
+		if (txp < 0 || txp > IWL_DEV_MAX_TX_POWER) {
+			err = -EINVAL;
+			goto free;
+		}
 		cmd.v5.v3.dev_52_low = cpu_to_le16(txp);
 	}
 
 	if (tb[IWL_MVM_VENDOR_ATTR_TXP_LIMIT_52H]) {
 		s32 txp = nla_get_u32(tb[IWL_MVM_VENDOR_ATTR_TXP_LIMIT_52H]);
 
-		if (txp < 0 || txp > IWL_DEV_MAX_TX_POWER)
-			return -EINVAL;
+		if (txp < 0 || txp > IWL_DEV_MAX_TX_POWER) {
+			err = -EINVAL;
+			goto free;
+		}
 		cmd.v5.v3.dev_52_high = cpu_to_le16(txp);
 	}
 
@@ -800,7 +525,10 @@ static int iwl_vendor_set_nic_txpower_limit(struct wiphy *wiphy,
 
 	if (err)
 		IWL_ERR(mvm, "failed to update device TX power: %d\n", err);
-	return 0;
+	err = 0;
+free:
+	kfree(tb);
+	return err;
 }
 
 #ifdef CPTCFG_IWLMVM_P2P_OPPPS_TEST_WA
@@ -831,16 +559,17 @@ static int iwl_mvm_oppps_wa(struct wiphy *wiphy,
 {
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
-	int err = iwl_mvm_parse_vendor_data(tb, data, data_len);
+	struct nlattr **tb;
+	int err;
 	struct ieee80211_vif *vif = wdev_to_ieee80211_vif(wdev);
 
-	if (err)
-		return err;
-
 	if (!vif)
 		return -ENODEV;
 
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
+
 	mutex_lock(&mvm->mutex);
 	if (vif->type == NL80211_IFTYPE_STATION && vif->p2p) {
 		bool enable = !!tb[IWL_MVM_VENDOR_ATTR_OPPPS_WA];
@@ -849,6 +578,7 @@ static int iwl_mvm_oppps_wa(struct wiphy *wiphy,
 	}
 	mutex_unlock(&mvm->mutex);
 
+	kfree(tb);
 	return err;
 }
 #endif
@@ -927,24 +657,28 @@ static int iwl_mvm_vendor_rxfilter(struct wiphy *wiphy,
 				   struct wireless_dev *wdev,
 				   const void *data, int data_len)
 {
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
 	enum iwl_mvm_vendor_rxfilter_flags filter, rx_filters, old_rx_filters;
 	enum iwl_mvm_vendor_rxfilter_op op;
 	bool first_set;
 	u32 mask;
-	int retval;
+	int err;
 
-	retval = iwl_mvm_parse_vendor_data(tb, data, data_len);
-	if (retval)
-		return retval;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
-	if (!tb[IWL_MVM_VENDOR_ATTR_RXFILTER])
-		return -EINVAL;
+	if (!tb[IWL_MVM_VENDOR_ATTR_RXFILTER]) {
+		err = -EINVAL;
+		goto free;
+	}
 
-	if (!tb[IWL_MVM_VENDOR_ATTR_RXFILTER_OP])
-		return -EINVAL;
+	if (!tb[IWL_MVM_VENDOR_ATTR_RXFILTER_OP]) {
+		err = -EINVAL;
+		goto free;
+	}
 
 	filter = nla_get_u32(tb[IWL_MVM_VENDOR_ATTR_RXFILTER]);
 	op = nla_get_u32(tb[IWL_MVM_VENDOR_ATTR_RXFILTER_OP]);
@@ -952,8 +686,10 @@ static int iwl_mvm_vendor_rxfilter(struct wiphy *wiphy,
 	if (filter != IWL_MVM_VENDOR_RXFILTER_UNICAST &&
 	    filter != IWL_MVM_VENDOR_RXFILTER_BCAST &&
 	    filter != IWL_MVM_VENDOR_RXFILTER_MCAST4 &&
-	    filter != IWL_MVM_VENDOR_RXFILTER_MCAST6)
-		return -EINVAL;
+	    filter != IWL_MVM_VENDOR_RXFILTER_MCAST6) {
+		err = -EINVAL;
+		goto free;
+	}
 
 	rx_filters = mvm->rx_filters & ~IWL_MVM_VENDOR_RXFILTER_EINVAL;
 	switch (op) {
@@ -964,7 +700,8 @@ static int iwl_mvm_vendor_rxfilter(struct wiphy *wiphy,
 		rx_filters |= filter;
 		break;
 	default:
-		return -EINVAL;
+		err = -EINVAL;
+		goto free;
 	}
 
 	first_set = mvm->rx_filters & IWL_MVM_VENDOR_RXFILTER_EINVAL;
@@ -972,8 +709,10 @@ static int iwl_mvm_vendor_rxfilter(struct wiphy *wiphy,
 	/* If first time set - clear EINVAL value */
 	mvm->rx_filters &= ~IWL_MVM_VENDOR_RXFILTER_EINVAL;
 
+	err = 0;
+
 	if (rx_filters == mvm->rx_filters && !first_set)
-		return 0;
+		goto free;
 
 	mutex_lock(&mvm->mutex);
 
@@ -992,7 +731,9 @@ static int iwl_mvm_vendor_rxfilter(struct wiphy *wiphy,
 
 	mutex_unlock(&mvm->mutex);
 
-	return 0;
+free:
+	kfree(tb);
+	return err;
 }
 
 static int iwl_mvm_vendor_dbg_collect(struct wiphy *wiphy,
@@ -1001,62 +742,78 @@ static int iwl_mvm_vendor_dbg_collect(struct wiphy *wiphy,
 {
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	int err, len = 0;
 	const char *trigger_desc;
 
-	err = iwl_mvm_parse_vendor_data(tb, data, data_len);
-	if (err)
-		return err;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
-	if (!tb[IWL_MVM_VENDOR_ATTR_DBG_COLLECT_TRIGGER])
-		return -EINVAL;
+	if (!tb[IWL_MVM_VENDOR_ATTR_DBG_COLLECT_TRIGGER]) {
+		err = -EINVAL;
+		goto free;
+	}
 
 	trigger_desc = nla_data(tb[IWL_MVM_VENDOR_ATTR_DBG_COLLECT_TRIGGER]);
 	len = nla_len(tb[IWL_MVM_VENDOR_ATTR_DBG_COLLECT_TRIGGER]);
 
 	iwl_fw_dbg_collect(&mvm->fwrt, FW_DBG_TRIGGER_USER_EXTENDED,
-			   trigger_desc, len);
+			   trigger_desc, len, NULL);
+	err = 0;
 
-	return 0;
+free:
+	kfree(tb);
+	return err;
 }
 
 static int iwl_mvm_vendor_nan_faw_conf(struct wiphy *wiphy,
 				       struct wireless_dev *wdev,
 				       const void *data, int data_len)
 {
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
 	struct cfg80211_chan_def def = {};
 	struct ieee80211_channel *chan;
 	u32 freq;
 	u8 slots;
-	int retval;
+	int err;
 
-	retval = iwl_mvm_parse_vendor_data(tb, data, data_len);
-	if (retval)
-		return retval;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
-	if (!tb[IWL_MVM_VENDOR_ATTR_NAN_FAW_SLOTS])
-		return -EINVAL;
+	if (!tb[IWL_MVM_VENDOR_ATTR_NAN_FAW_SLOTS]) {
+		err = -EINVAL;
+		goto free;
+	}
 
-	if (!tb[IWL_MVM_VENDOR_ATTR_NAN_FAW_FREQ])
-		return -EINVAL;
+	if (!tb[IWL_MVM_VENDOR_ATTR_NAN_FAW_FREQ]) {
+		err = -EINVAL;
+		goto free;
+	}
 
 	freq = nla_get_u32(tb[IWL_MVM_VENDOR_ATTR_NAN_FAW_FREQ]);
 	slots = nla_get_u8(tb[IWL_MVM_VENDOR_ATTR_NAN_FAW_SLOTS]);
 
 	chan = ieee80211_get_channel(wiphy, freq);
-	if (!chan)
-		return -EINVAL;
+	if (!chan) {
+		err = -EINVAL;
+		goto free;
+	}
 
 	cfg80211_chandef_create(&def, chan, NL80211_CHAN_NO_HT);
 
-	if (!cfg80211_chandef_usable(wiphy, &def, IEEE80211_CHAN_DISABLED))
-		return -EINVAL;
+	if (!cfg80211_chandef_usable(wiphy, &def, IEEE80211_CHAN_DISABLED)) {
+		err = -EINVAL;
+		goto free;
+	}
 
-	return iwl_mvm_nan_config_nan_faw_cmd(mvm, &def, slots);
+	err = iwl_mvm_nan_config_nan_faw_cmd(mvm, &def, slots);
+free:
+	kfree(tb);
+	return err;
 }
 
 #ifdef CONFIG_ACPI
@@ -1067,29 +824,38 @@ static int iwl_mvm_vendor_set_dynamic_txp_profile(struct wiphy *wiphy,
 {
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
-	int ret;
+	struct nlattr **tb;
 	u8 chain_a, chain_b;
+	int err;
 
-	ret = iwl_mvm_parse_vendor_data(tb, data, data_len);
-	if (ret)
-		return ret;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
 	if (!tb[IWL_MVM_VENDOR_ATTR_SAR_CHAIN_A_PROFILE] ||
-	    !tb[IWL_MVM_VENDOR_ATTR_SAR_CHAIN_B_PROFILE])
-		return -EINVAL;
+	    !tb[IWL_MVM_VENDOR_ATTR_SAR_CHAIN_B_PROFILE]) {
+		err = -EINVAL;
+		goto free;
+	}
 
 	chain_a = nla_get_u8(tb[IWL_MVM_VENDOR_ATTR_SAR_CHAIN_A_PROFILE]);
 	chain_b = nla_get_u8(tb[IWL_MVM_VENDOR_ATTR_SAR_CHAIN_B_PROFILE]);
 
 	if (mvm->sar_chain_a_profile == chain_a &&
-	    mvm->sar_chain_b_profile == chain_b)
-		return 0;
+	    mvm->sar_chain_b_profile == chain_b) {
+		err = 0;
+		goto free;
+	}
 
 	mvm->sar_chain_a_profile = chain_a;
 	mvm->sar_chain_b_profile = chain_b;
 
-	return iwl_mvm_sar_select_profile(mvm, chain_a, chain_b);
+	mutex_lock(&mvm->mutex);
+	err = iwl_mvm_sar_select_profile(mvm, chain_a, chain_b);
+	mutex_unlock(&mvm->mutex);
+free:
+	kfree(tb);
+	return err;
 }
 
 static int iwl_mvm_vendor_get_sar_profile_info(struct wiphy *wiphy,
@@ -1351,7 +1117,7 @@ static int iwl_mvm_vendor_test_fips(struct wiphy *wiphy,
 				    struct wireless_dev *wdev,
 				    const void *data, int data_len)
 {
-	struct nlattr *tb[NUM_IWL_MVM_VENDOR_ATTR];
+	struct nlattr **tb;
 	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
 	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
 	struct iwl_host_cmd hcmd = {
@@ -1366,9 +1132,9 @@ static int iwl_mvm_vendor_test_fips(struct wiphy *wiphy,
 	u8 *buf = NULL;
 	int ret;
 
-	ret = iwl_mvm_parse_vendor_data(tb, data, data_len);
-	if (ret)
-		return ret;
+	tb = iwl_mvm_parse_vendor_data(data, data_len);
+	if (IS_ERR(tb))
+		return PTR_ERR(tb);
 
 	if (tb[IWL_MVM_VENDOR_ATTR_FIPS_TEST_VECTOR_HW_CCM]) {
 		vector = tb[IWL_MVM_VENDOR_ATTR_FIPS_TEST_VECTOR_HW_CCM];
@@ -1380,12 +1146,13 @@ static int iwl_mvm_vendor_test_fips(struct wiphy *wiphy,
 		vector = tb[IWL_MVM_VENDOR_ATTR_FIPS_TEST_VECTOR_HW_AES];
 		flags = IWL_FIPS_TEST_VECTOR_FLAGS_AES;
 	} else {
-		return -EINVAL;
+		ret = -EINVAL;
+		goto free;
 	}
 
 	ret = iwl_mvm_vendor_build_vector(&buf, vector, flags);
 	if (ret <= 0)
-		return ret;
+		goto free;
 
 	hcmd.data[0] = buf;
 	hcmd.len[0] = ret;
@@ -1405,9 +1172,22 @@ static int iwl_mvm_vendor_test_fips(struct wiphy *wiphy,
 
 free:
 	kfree(buf);
+	kfree(tb);
 	return ret;
 }
 
+static int iwl_mvm_vendor_csi_register(struct wiphy *wiphy,
+				       struct wireless_dev *wdev,
+				       const void *data, int data_len)
+{
+	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
+	struct iwl_mvm *mvm = IWL_MAC80211_GET_MVM(hw);
+
+	mvm->csi_portid = cfg80211_vendor_cmd_get_sender(wiphy);
+
+	return 0;
+}
+
 static const struct wiphy_vendor_command iwl_mvm_vendor_commands[] = {
 	{
 		.info = {
@@ -1427,53 +1207,6 @@ static const struct wiphy_vendor_command iwl_mvm_vendor_commands[] = {
 			 WIPHY_VENDOR_CMD_NEED_RUNNING,
 		.doit = iwl_mvm_get_low_latency,
 	},
-#ifdef CPTCFG_IWLWIFI_LTE_COEX
-	{
-		.info = {
-			.vendor_id = INTEL_OUI,
-			.subcmd = IWL_MVM_VENDOR_CMD_LTE_STATE,
-		},
-		.flags = WIPHY_VENDOR_CMD_NEED_NETDEV |
-			 WIPHY_VENDOR_CMD_NEED_RUNNING,
-		.doit = iwl_vendor_lte_coex_state_cmd,
-	},
-	{
-		.info = {
-			.vendor_id = INTEL_OUI,
-			.subcmd = IWL_MVM_VENDOR_CMD_LTE_COEX_CONFIG_INFO,
-		},
-		.flags = WIPHY_VENDOR_CMD_NEED_NETDEV |
-			 WIPHY_VENDOR_CMD_NEED_RUNNING,
-		.doit = iwl_vendor_lte_coex_config_cmd,
-	},
-	{
-		.info = {
-			.vendor_id = INTEL_OUI,
-			.subcmd = IWL_MVM_VENDOR_CMD_LTE_COEX_DYNAMIC_INFO,
-		},
-		.flags = WIPHY_VENDOR_CMD_NEED_NETDEV |
-			 WIPHY_VENDOR_CMD_NEED_RUNNING,
-		.doit = iwl_vendor_lte_coex_dynamic_info_cmd,
-	},
-	{
-		.info = {
-			.vendor_id = INTEL_OUI,
-			.subcmd = IWL_MVM_VENDOR_CMD_LTE_COEX_SPS_INFO,
-		},
-		.flags = WIPHY_VENDOR_CMD_NEED_NETDEV |
-			 WIPHY_VENDOR_CMD_NEED_RUNNING,
-		.doit = iwl_vendor_lte_sps_cmd,
-	},
-	{
-		.info = {
-			.vendor_id = INTEL_OUI,
-			.subcmd = IWL_MVM_VENDOR_CMD_LTE_COEX_WIFI_RPRTD_CHAN,
-		},
-		.flags = WIPHY_VENDOR_CMD_NEED_NETDEV |
-			 WIPHY_VENDOR_CMD_NEED_RUNNING,
-		.doit = iwl_vendor_lte_coex_wifi_reported_channel_cmd,
-	},
-#endif /* CPTCFG_IWLWIFI_LTE_COEX */
 	{
 		.info = {
 			.vendor_id = INTEL_OUI,
@@ -1605,10 +1338,18 @@ static const struct wiphy_vendor_command iwl_mvm_vendor_commands[] = {
 		.doit = iwl_mvm_vendor_test_fips,
 	},
 #endif
+	{
+		.info = {
+			.vendor_id = INTEL_OUI,
+			.subcmd = IWL_MVM_VENDOR_CMD_CSI_EVENT,
+		},
+		.doit = iwl_mvm_vendor_csi_register,
+	},
 };
 
 enum iwl_mvm_vendor_events_idx {
 	IWL_MVM_VENDOR_EVENT_IDX_TCM,
+	IWL_MVM_VENDOR_EVENT_IDX_CSI,
 	NUM_IWL_MVM_VENDOR_EVENT_IDX
 };
 
@@ -1618,14 +1359,29 @@ iwl_mvm_vendor_events[NUM_IWL_MVM_VENDOR_EVENT_IDX] = {
 		.vendor_id = INTEL_OUI,
 		.subcmd = IWL_MVM_VENDOR_CMD_TCM_EVENT,
 	},
+	[IWL_MVM_VENDOR_EVENT_IDX_CSI] = {
+		.vendor_id = INTEL_OUI,
+		.subcmd = IWL_MVM_VENDOR_CMD_CSI_EVENT,
+	},
 };
 
-void iwl_mvm_set_wiphy_vendor_commands(struct wiphy *wiphy)
+void iwl_mvm_vendor_cmds_register(struct iwl_mvm *mvm)
 {
-	wiphy->vendor_commands = iwl_mvm_vendor_commands;
-	wiphy->n_vendor_commands = ARRAY_SIZE(iwl_mvm_vendor_commands);
-	wiphy->vendor_events = iwl_mvm_vendor_events;
-	wiphy->n_vendor_events = ARRAY_SIZE(iwl_mvm_vendor_events);
+	mvm->hw->wiphy->vendor_commands = iwl_mvm_vendor_commands;
+	mvm->hw->wiphy->n_vendor_commands = ARRAY_SIZE(iwl_mvm_vendor_commands);
+	mvm->hw->wiphy->vendor_events = iwl_mvm_vendor_events;
+	mvm->hw->wiphy->n_vendor_events = ARRAY_SIZE(iwl_mvm_vendor_events);
+
+	spin_lock_bh(&device_list_lock);
+	list_add_tail(&mvm->list, &device_list);
+	spin_unlock_bh(&device_list_lock);
+}
+
+void iwl_mvm_vendor_cmds_unregister(struct iwl_mvm *mvm)
+{
+	spin_lock_bh(&device_list_lock);
+	list_del(&mvm->list);
+	spin_unlock_bh(&device_list_lock);
 }
 
 static enum iwl_mvm_vendor_load
@@ -1679,3 +1435,173 @@ void iwl_mvm_send_tcm_event(struct iwl_mvm *mvm, struct ieee80211_vif *vif)
  nla_put_failure:
 	kfree_skb(msg);
 }
+
+static void
+iwl_mvm_send_csi_event(struct iwl_mvm *mvm,
+		       void *hdr, unsigned int hdr_len,
+		       void **data, unsigned int *len)
+{
+	unsigned int data_len = 0;
+	struct sk_buff *msg;
+	struct nlattr *dattr;
+	u8 *pos;
+	int i;
+
+	if (!mvm->csi_portid)
+		return;
+
+	for (i = 0; len[i] && data[i]; i++)
+		data_len += len[i];
+
+	msg = cfg80211_vendor_event_alloc_ucast(mvm->hw->wiphy, NULL,
+						mvm->csi_portid,
+						100 + hdr_len + data_len,
+						IWL_MVM_VENDOR_EVENT_IDX_CSI,
+						GFP_KERNEL);
+
+	if (!msg)
+		return;
+
+	if (nla_put(msg, IWL_MVM_VENDOR_ATTR_CSI_HDR, hdr_len, hdr))
+		goto nla_put_failure;
+
+	dattr = nla_reserve(msg, IWL_MVM_VENDOR_ATTR_CSI_DATA, data_len);
+	if (!dattr)
+		goto nla_put_failure;
+
+	pos = nla_data(dattr);
+	for (i = 0; len[i] && data[i]; i++) {
+		memcpy(pos, data[i], len[i]);
+		pos += len[i];
+	}
+
+	cfg80211_vendor_event(msg, GFP_KERNEL);
+	return;
+
+ nla_put_failure:
+	kfree_skb(msg);
+}
+
+static void iwl_mvm_csi_free_pages(struct iwl_mvm *mvm)
+{
+	int idx;
+
+	for (idx = 0; idx < ARRAY_SIZE(mvm->csi_data_entries); idx++) {
+		if (mvm->csi_data_entries[idx].page) {
+			__free_pages(mvm->csi_data_entries[idx].page,
+				     mvm->csi_data_entries[idx].page_order);
+			memset(&mvm->csi_data_entries[idx], 0,
+			       sizeof(mvm->csi_data_entries[idx]));
+		}
+	}
+}
+
+static void iwl_mvm_csi_steal(struct iwl_mvm *mvm, unsigned int idx,
+			      struct iwl_rx_cmd_buffer *rxb)
+{
+	/* firmware is ... confused */
+	if (WARN_ON(mvm->csi_data_entries[idx].page)) {
+		iwl_mvm_csi_free_pages(mvm);
+		return;
+	}
+
+	mvm->csi_data_entries[idx].page = rxb_steal_page(rxb);
+	mvm->csi_data_entries[idx].page_order = rxb->_rx_page_order;
+	mvm->csi_data_entries[idx].offset = rxb->_offset;
+}
+
+void iwl_mvm_rx_csi_header(struct iwl_mvm *mvm, struct iwl_rx_cmd_buffer *rxb)
+{
+	iwl_mvm_csi_steal(mvm, 0, rxb);
+}
+
+static void iwl_mvm_csi_complete(struct iwl_mvm *mvm)
+{
+	struct iwl_rx_packet *hdr_pkt;
+	struct iwl_csi_data_buffer *hdr_buf = &mvm->csi_data_entries[0];
+	void *data[IWL_CSI_MAX_EXPECTED_CHUNKS + 1] = {};
+	unsigned int len[IWL_CSI_MAX_EXPECTED_CHUNKS + 1] = {};
+	unsigned int csi_hdr_len;
+	void *csi_hdr;
+	int i;
+
+	/*
+	 * Ensure we have the right # of entries, the local data/len
+	 * variables include a terminating entry, csi_data_entries
+	 * instead has one place for the header.
+	 */
+	BUILD_BUG_ON(ARRAY_SIZE(data) < ARRAY_SIZE(mvm->csi_data_entries));
+	BUILD_BUG_ON(ARRAY_SIZE(len) < ARRAY_SIZE(mvm->csi_data_entries));
+
+	/* need at least the header and first fragment */
+	if (WARN_ON(!hdr_buf->page || !mvm->csi_data_entries[1].page)) {
+		iwl_mvm_csi_free_pages(mvm);
+		return;
+	}
+
+	hdr_pkt = (void *)((unsigned long)page_address(hdr_buf->page) +
+			   hdr_buf->offset);
+	csi_hdr = (void *)hdr_pkt->data;
+	csi_hdr_len = iwl_rx_packet_payload_len(hdr_pkt);
+
+	for (i = 1; i < ARRAY_SIZE(mvm->csi_data_entries); i++) {
+		struct iwl_csi_data_buffer *buf = &mvm->csi_data_entries[i];
+		struct iwl_rx_packet *pkt;
+		struct iwl_csi_chunk_notification *chunk;
+		unsigned int chunk_len;
+
+		if (!buf->page)
+			break;
+
+		pkt = (void *)((unsigned long)page_address(buf->page) +
+			       buf->offset);
+		chunk = (void *)pkt->data;
+		chunk_len = iwl_rx_packet_payload_len(pkt);
+
+		if (sizeof(*chunk) + le32_to_cpu(chunk->size) > chunk_len)
+			goto free;
+
+		data[i - 1] = chunk->data;
+		len[i - 1] = le32_to_cpu(chunk->size);
+	}
+
+	iwl_mvm_send_csi_event(mvm, csi_hdr, csi_hdr_len, data, len);
+
+ free:
+	iwl_mvm_csi_free_pages(mvm);
+}
+
+void iwl_mvm_rx_csi_chunk(struct iwl_mvm *mvm, struct iwl_rx_cmd_buffer *rxb)
+{
+	struct iwl_rx_packet *pkt = rxb_addr(rxb);
+	struct iwl_csi_chunk_notification *chunk = (void *)pkt->data;
+	int num;
+	int idx;
+
+	switch (mvm->cmd_ver.csi_notif) {
+	case 1:
+		num = le16_get_bits(chunk->ctl,
+				    IWL_CSI_CHUNK_CTL_NUM_MASK_VER_1);
+		idx = le16_get_bits(chunk->ctl,
+				    IWL_CSI_CHUNK_CTL_IDX_MASK_VER_1) + 1;
+		break;
+	case 2:
+		num = le16_get_bits(chunk->ctl,
+				    IWL_CSI_CHUNK_CTL_NUM_MASK_VER_2);
+		idx = le16_get_bits(chunk->ctl,
+				    IWL_CSI_CHUNK_CTL_IDX_MASK_VER_2) + 1;
+		break;
+	default:
+		WARN_ON(1);
+		return;
+	}
+
+	/* -1 to account for the header we also store there */
+	if (WARN_ON_ONCE(idx >= ARRAY_SIZE(mvm->csi_data_entries) - 1))
+		return;
+
+	iwl_mvm_csi_steal(mvm, idx, rxb);
+
+	if (num == idx)
+		iwl_mvm_csi_complete(mvm);
+}
