@@ -7,7 +7,7 @@
  *
  * Copyright(c) 2007 - 2014 Intel Corporation. All rights reserved.
  * Copyright(c) 2015 - 2017 Intel Deutschland GmbH
- * Copyright(c) 2018        Intel Corporation
+ * Copyright(c) 2018 - 2019 Intel Corporation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -29,7 +29,7 @@
  *
  * Copyright(c) 2005 - 2014 Intel Corporation. All rights reserved.
  * Copyright(c) 2017   Intel Deutschland GmbH
- * Copyright(c) 2018   Intel Corporation
+ * Copyright(c) 2018 - 2019 Intel Corporation
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -125,6 +125,7 @@ static const struct iwl_hcmd_names iwl_xvt_cmd_names[] = {
 	HCMD_NAME(BA_NOTIF),
 	HCMD_NAME(DTS_MEASUREMENT_NOTIFICATION),
 	HCMD_NAME(REPLY_DEBUG_XVT_CMD),
+	HCMD_NAME(LDBG_CONFIG_CMD),
 	HCMD_NAME(DEBUG_LOG_MSG),
 };
 
@@ -170,7 +171,7 @@ static const struct iwl_hcmd_names iwl_xvt_regulatory_and_nvm_names[] = {
 /* Please keep this array *SORTED* by hex value.
  * Access is done through binary search.
  */
-static const struct iwl_hcmd_names iwl_xvt_tof_names[] = {
+static const struct iwl_hcmd_names iwl_xvt_location_names[] = {
 	HCMD_NAME(LOCATION_GROUP_NOTIFICATION),
 	HCMD_NAME(TOF_MCSI_DEBUG_NOTIF),
 	HCMD_NAME(TOF_RANGE_RESPONSE_NOTIF),
@@ -187,15 +188,20 @@ static const struct iwl_hcmd_names iwl_xvt_xvt_names[] = {
 	HCMD_NAME(IQ_CALIB_CONFIG_NOTIF),
 };
 
+static const struct iwl_hcmd_names iwl_xvt_debug_names[] = {
+	HCMD_NAME(DBGC_SUSPEND_RESUME),
+};
+
 static const struct iwl_hcmd_arr iwl_xvt_cmd_groups[] = {
 	[LEGACY_GROUP] = HCMD_ARR(iwl_xvt_cmd_names),
 	[LONG_GROUP] = HCMD_ARR(iwl_xvt_long_cmd_names),
 	[SYSTEM_GROUP] = HCMD_ARR(iwl_xvt_system_names),
 	[PHY_OPS_GROUP] = HCMD_ARR(iwl_xvt_phy_names),
 	[DATA_PATH_GROUP] = HCMD_ARR(iwl_xvt_data_path_names),
-	[TOF_GROUP] = HCMD_ARR(iwl_xvt_tof_names),
+	[LOCATION_GROUP] = HCMD_ARR(iwl_xvt_location_names),
 	[REGULATORY_AND_NVM_GROUP] = HCMD_ARR(iwl_xvt_regulatory_and_nvm_names),
 	[XVT_GROUP] = HCMD_ARR(iwl_xvt_xvt_names),
+	[DEBUG_GROUP] = HCMD_ARR(iwl_xvt_debug_names),
 };
 
 static int iwl_xvt_tm_send_hcmd(void *op_mode, struct iwl_host_cmd *host_cmd)
@@ -252,7 +258,8 @@ static struct iwl_op_mode *iwl_xvt_start(struct iwl_trans *trans,
 	trans_cfg.cmd_queue = IWL_MVM_DQA_CMD_QUEUE;
 	IWL_DEBUG_INFO(xvt, "dqa supported\n");
 	trans_cfg.cmd_fifo = IWL_MVM_TX_FIFO_CMD;
-	trans_cfg.bc_table_dword = true;
+	trans_cfg.bc_table_dword =
+		trans->cfg->device_family < IWL_DEVICE_FAMILY_22560;
 	trans_cfg.scd_set_active = true;
 	trans->wide_cmd_header = true;
 
@@ -318,11 +325,11 @@ static struct iwl_op_mode *iwl_xvt_start(struct iwl_trans *trans,
 	memset(xvt->queue_data, 0, sizeof(xvt->queue_data));
 	init_waitqueue_head(&xvt->tx_done_wq);
 
-	trans->dbg_dest_tlv = xvt->fw->dbg.dest_tlv;
-	trans->dbg_n_dest_reg = xvt->fw->dbg.n_dest_reg;
-	memcpy(trans->dbg_conf_tlv, xvt->fw->dbg.conf_tlv,
-	       sizeof(trans->dbg_conf_tlv));
-	trans->dbg_trigger_tlv = xvt->fw->dbg.trigger_tlv;
+	trans->dbg.dest_tlv = xvt->fw->dbg.dest_tlv;
+	trans->dbg.n_dest_reg = xvt->fw->dbg.n_dest_reg;
+	memcpy(trans->dbg.conf_tlv, xvt->fw->dbg.conf_tlv,
+	       sizeof(trans->dbg.conf_tlv));
+	trans->dbg.trigger_tlv = xvt->fw->dbg.trigger_tlv;
 
 	IWL_INFO(xvt, "Detected %s, REV=0x%X, xVT operation mode\n",
 		 xvt->cfg->name, xvt->trans->hw_rev);
@@ -351,6 +358,7 @@ static void iwl_xvt_stop(struct iwl_op_mode *op_mode)
 			iwl_xvt_txq_disable(xvt);
 			xvt->fw_running = false;
 		}
+		iwl_fw_dbg_stop_sync(&xvt->fwrt);
 		iwl_trans_stop_device(xvt->trans);
 	}
 
@@ -640,7 +648,9 @@ static void iwl_xvt_nic_error(struct iwl_op_mode *op_mode)
 	p_table = kmemdup(&table_v2, sizeof(table_v2), GFP_ATOMIC);
 	table_size = sizeof(table_v2);
 
-	if (xvt->support_umac_log) {
+	if (xvt->support_umac_log ||
+	    (xvt->trans->dbg.error_event_table_tlv_status &
+	     IWL_ERROR_EVENT_TABLE_UMAC)) {
 		iwl_xvt_get_umac_error_log(xvt, &table_umac);
 		iwl_xvt_dump_umac_error_log(xvt, &table_umac);
 		p_table_umac = kmemdup(&table_umac, sizeof(table_umac),
@@ -670,7 +680,7 @@ static void iwl_xvt_nic_error(struct iwl_op_mode *op_mode)
 		kfree(p_table_umac);
 	}
 
-	iwl_fw_dbg_collect_desc(&xvt->fwrt, &iwl_dump_desc_assert, false, 0);
+	iwl_fw_error_collect(&xvt->fwrt);
 }
 
 static bool iwl_xvt_set_hw_rfkill_state(struct iwl_op_mode *op_mode, bool state)
@@ -763,12 +773,13 @@ void iwl_xvt_free_tx_queue(struct iwl_xvt *xvt, u8 lmac_id)
 int iwl_xvt_allocate_tx_queue(struct iwl_xvt *xvt, u8 sta_id,
 			      u8 lmac_id)
 {
-	int ret;
+	int ret, size = max_t(u32, IWL_DEFAULT_QUEUE_SIZE,
+			      xvt->trans->cfg->min_256_ba_txq_size);
 
 	ret = iwl_trans_txq_alloc(xvt->trans,
 				  cpu_to_le16(TX_QUEUE_CFG_ENABLE_QUEUE),
 				  sta_id, TX_QUEUE_CFG_TID, SCD_QUEUE_CFG,
-				  IWL_DEFAULT_QUEUE_SIZE, 0);
+				  size, 0);
 	/* ret is positive when func returns the allocated the queue number */
 	if (ret > 0) {
 		xvt->tx_meta_data[lmac_id].queue = ret;
